@@ -1,21 +1,7 @@
 /*
- *   Copyright (C) 2012 - 2016 by Ivan Cukic <ivan.cukic@kde.org>
+ *   SPDX-FileCopyrightText: 2012-2016 Ivan Cukic <ivan.cukic@kde.org>
  *
- *   This program is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU General Public License as
- *   published by the Free Software Foundation; either version 2 of
- *   the License or (at your option) version 3 or any later version
- *   accepted by the membership of KDE e.V. (or its successor approved
- *   by the membership of KDE e.V.), which shall act as a proxy
- *   defined in Section 14 of version 3 of the license.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "KioActivities.h"
@@ -35,7 +21,7 @@
 
 #include <KLocalizedString>
 #include <KUser>
-#include <kio/job.h>
+#include <KIO/Job>
 
 #include <utils/d_ptr_implementation.h>
 #include <utils/qsqlquery_iterator.h>
@@ -46,6 +32,13 @@
 
 #include <QProcess>
 
+// Pseudo plugin class to embed meta data
+class KIOPluginForMetaData : public QObject
+{
+    Q_OBJECT
+    Q_PLUGIN_METADATA(IID "org.kde.kio.slave.activities" FILE "activities.json")
+};
+
 class ActivitiesProtocol::Private {
 public:
     Private()
@@ -62,15 +55,15 @@ public:
                       QString *filePath = nullptr) const
     {
         const auto fullPath = url.adjusted(QUrl::StripTrailingSlash).path();
-        const auto path = fullPath.midRef(fullPath.startsWith('/') ? 1 : 0);
+        const auto path = QStringView(fullPath).mid(fullPath.startsWith(QLatin1Char('/')) ? 1 : 0);
 
         if (activity) {
-            *activity = path.mid(0, path.indexOf("/") - 1).toString();
+            *activity = path.mid(0, path.indexOf(QStringLiteral("/")) - 1).toString();
         }
 
         if (filePath) {
-            auto strippedPath = path.mid(path.indexOf("/") + 1);
-            auto splitPosition = strippedPath.indexOf("/");
+            auto strippedPath = path.mid(path.indexOf(QStringLiteral("/")) + 1);
+            auto splitPosition = strippedPath.indexOf(QStringLiteral("/"));
 
             if (splitPosition == -1) {
                 // if we have only one path segment
@@ -86,8 +79,8 @@ public:
         }
 
         return path.length() == 0 ? RootItem
-             : path.contains("/") ? ActivityPathItem
-             : ActivityRootItem;
+               : path.contains(QStringLiteral("/")) ? ActivityPathItem
+               : ActivityRootItem;
     }
 
     void syncActivities(KActivities::Consumer &activities)
@@ -142,14 +135,14 @@ public:
     {
         // return QString::fromUtf8(QUrl::toPercentEncoding(path));
         return QString::fromLatin1(path.toUtf8().toBase64(
-            QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals));
+                                       QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals));
     }
 
     QString demangledPath(const QString &mangled) const
     {
         // return QUrl::fromPercentEncoding(mangled.toUtf8());
         return QString::fromUtf8(QByteArray::fromBase64(mangled.toLatin1(),
-            QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals));
+                                 QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals));
     }
 
     // KActivities::Consumer activities;
@@ -186,30 +179,30 @@ bool ActivitiesProtocol::rewriteUrl(const QUrl &url, QUrl &newUrl)
 {
     QString activity, path;
     switch (d->pathType(url, &activity, &path)) {
-        case Private::RootItem:
-        case Private::ActivityRootItem:
-            if (activity == "current") {
-                KActivities::Consumer activities;
-                d->syncActivities(activities);
-                newUrl = QUrl(QStringLiteral("activities:/")
-                                  + activities.currentActivity());
-                return true;
-            }
-            return false;
-
-        case Private::ActivityPathItem:
-        {
-            // auto demangled = d->demangledPath(path);
-            // QProcess::execute("kdialog",
-            //                   { "--passivepopup",
-            //                     path.midRef(1).toString() + "\n" + demangled });
-
-            newUrl = QUrl::fromLocalFile(path);
+    case Private::RootItem:
+    case Private::ActivityRootItem:
+        if (activity == "current") {
+            KActivities::Consumer activities;
+            d->syncActivities(activities);
+            newUrl = QUrl(QStringLiteral("activities:/")
+                          + activities.currentActivity());
             return true;
         }
+        return false;
 
-        default:
-            return true;
+    case Private::ActivityPathItem:
+    {
+        // auto demangled = d->demangledPath(path);
+        // QProcess::execute("kdialog",
+        //                   { "--passivepopup",
+        //                     path.midRef(1).toString() + "\n" + demangled });
+
+        newUrl = QUrl::fromLocalFile(path);
+        return true;
+    }
+
+    default:
+        return true;
     }
 }
 
@@ -220,76 +213,76 @@ void ActivitiesProtocol::listDir(const QUrl &url)
 
     QString activity, path;
     switch (d->pathType(url, &activity, &path)) {
-        case Private::RootItem:
-        {
-            KIO::UDSEntryList udslist;
+    case Private::RootItem:
+    {
+        KIO::UDSEntryList udslist;
+
+        KIO::UDSEntry uds;
+        uds.reserve(9);
+        uds.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("current"));
+        uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_NAME, i18n("Current activity"));
+        uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_TYPE, i18n("Activity"));
+        uds.fastInsert(KIO::UDSEntry::UDS_ICON_NAME, QStringLiteral("activities"));
+        uds.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+        uds.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
+        uds.fastInsert(KIO::UDSEntry::UDS_ACCESS, 0500);
+        uds.fastInsert(KIO::UDSEntry::UDS_USER, KUser().loginName());
+        uds.fastInsert(KIO::UDSEntry::UDS_TARGET_URL, QStringLiteral("activities:/") + activities.currentActivity());
+        udslist << uds;
+
+        for (const auto& activity: activities.activities()) {
+            udslist << d->activityEntry(activity);
+        }
+
+        listEntries(udslist);
+        finished();
+        break;
+    }
+
+    case Private::ActivityRootItem:
+    {
+        KIO::UDSEntryList udslist;
+
+        auto database = Common::Database::instance(
+                            Common::Database::ResourcesDatabase,
+                            Common::Database::ReadOnly);
+
+        if (!database) {
+            finished();
+            break;
+        }
+
+        if (activity == "current") {
+            activity = activities.currentActivity();
+        }
+
+        static const auto queryString = QStringLiteral(
+                                            "SELECT targettedResource "
+                                            "FROM ResourceLink "
+                                            "WHERE usedActivity = '%1' "
+                                            "AND initiatingAgent = \":global\" "
+                                        );
+
+        auto query = database->execQuery(queryString.arg(activity));
+
+        for (const auto& result: query) {
+            auto path = result[0].toString();
+
+            if (!QFile(path).exists()) continue;
 
             KIO::UDSEntry uds;
-            uds.reserve(9);
-            uds.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("current"));
-            uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_NAME, i18n("Current activity"));
-            uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_TYPE, i18n("Activity"));
-            uds.fastInsert(KIO::UDSEntry::UDS_ICON_NAME, QStringLiteral("activities"));
-            uds.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-            uds.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
-            uds.fastInsert(KIO::UDSEntry::UDS_ACCESS, 0500);
-            uds.fastInsert(KIO::UDSEntry::UDS_USER, KUser().loginName());
-            uds.fastInsert(KIO::UDSEntry::UDS_TARGET_URL, QStringLiteral("activities:/") + activities.currentActivity());
-            udslist << uds;
 
-            for (const auto& activity: activities.activities()) {
-                udslist << d->activityEntry(activity);
-            }
-
-            listEntries(udslist);
-            finished();
-            break;
+            udslist << d->filesystemEntry(path);
         }
 
-        case Private::ActivityRootItem:
-        {
-            KIO::UDSEntryList udslist;
+        listEntries(udslist);
+        finished();
+        break;
+    }
 
-            auto database = Common::Database::instance(
-                Common::Database::ResourcesDatabase,
-                Common::Database::ReadOnly);
-
-            if (!database) {
-                finished();
-                break;
-            }
-
-            if (activity == "current") {
-                activity = activities.currentActivity();
-            }
-
-            static const auto queryString = QStringLiteral(
-                "SELECT targettedResource "
-                "FROM ResourceLink "
-                "WHERE usedActivity = '%1' "
-                    "AND initiatingAgent = \":global\" "
-                );
-
-            auto query = database->execQuery(queryString.arg(activity));
-
-            for (const auto& result: query) {
-                auto path = result[0].toString();
-
-                if (!QFile(path).exists()) continue;
-
-                KIO::UDSEntry uds;
-
-                udslist << d->filesystemEntry(path);
-            }
-
-            listEntries(udslist);
-            finished();
-            break;
-        }
-
-        case Private::ActivityPathItem:
-            ForwardingSlaveBase::listDir(QUrl::fromLocalFile(path));
-            break;
+    case Private::ActivityPathItem:
+        ForwardingSlaveBase::listDir(QUrl::fromLocalFile(path));
+        break;
     }
 }
 
@@ -303,55 +296,55 @@ void ActivitiesProtocol::stat(const QUrl& url)
     QString activity;
 
     switch (d->pathType(url, &activity)) {
-        case Private::RootItem:
-        {
-            QString dirName = i18n("Activities");
-            KIO::UDSEntry uds;
-            uds.reserve(6);
-            uds.fastInsert(KIO::UDSEntry::UDS_NAME, dirName);
-            uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_NAME, dirName);
-            uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_TYPE, dirName);
-            uds.fastInsert(KIO::UDSEntry::UDS_ICON_NAME, QStringLiteral("activities"));
-            uds.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-            uds.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
+    case Private::RootItem:
+    {
+        QString dirName = i18n("Activities");
+        KIO::UDSEntry uds;
+        uds.reserve(6);
+        uds.fastInsert(KIO::UDSEntry::UDS_NAME, dirName);
+        uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_NAME, dirName);
+        uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_TYPE, dirName);
+        uds.fastInsert(KIO::UDSEntry::UDS_ICON_NAME, QStringLiteral("activities"));
+        uds.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+        uds.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
 
-            statEntry(uds);
-            finished();
-            break;
+        statEntry(uds);
+        finished();
+        break;
+    }
+
+    case Private::ActivityRootItem:
+    {
+        KActivities::Consumer activities;
+        d->syncActivities(activities);
+
+        if (activity == "current") {
+            activity = activities.currentActivity();
         }
 
-        case Private::ActivityRootItem:
-        {
-            KActivities::Consumer activities;
-            d->syncActivities(activities);
+        statEntry(d->activityEntry(activity));
+        finished();
+        break;
+    }
 
-            if (activity == "current") {
-                activity = activities.currentActivity();
-            }
-
-            statEntry(d->activityEntry(activity));
-            finished();
-            break;
-        }
-
-        case Private::ActivityPathItem:
-            ForwardingSlaveBase::stat(url);
-            break;
+    case Private::ActivityPathItem:
+        ForwardingSlaveBase::stat(url);
+        break;
     }
 }
 
 void ActivitiesProtocol::mimetype(const QUrl& url)
 {
     switch (d->pathType(url)) {
-        case Private::RootItem:
-        case Private::ActivityRootItem:
-            mimeType(QStringLiteral("inode/directory"));
-            finished();
-            break;
+    case Private::RootItem:
+    case Private::ActivityRootItem:
+        mimeType(QStringLiteral("inode/directory"));
+        finished();
+        break;
 
-        case Private::ActivityPathItem:
-            ForwardingSlaveBase::mimetype(url);
-            break;
+    case Private::ActivityPathItem:
+        ForwardingSlaveBase::mimetype(url);
+        break;
     }
 
 }
@@ -362,4 +355,4 @@ void ActivitiesProtocol::del(const QUrl& url, bool isfile)
     Q_UNUSED(isfile);
 }
 
-
+#include "KioActivities.moc"
