@@ -21,12 +21,7 @@
 #include <KLocalizedString>
 
 #include "man2html.h"
-#include <karchive_version.h>
-#if KARCHIVE_VERSION >= QT_VERSION_CHECK(5, 85, 0)
 #include <KCompressionDevice>
-#else
-#include <KFilterDev>
-#endif
 
 
 using namespace KIO;
@@ -35,7 +30,7 @@ using namespace KIO;
 class KIOPluginForMetaData : public QObject
 {
     Q_OBJECT
-    Q_PLUGIN_METADATA(IID "org.kde.kio.slave.man" FILE "man.json")
+    Q_PLUGIN_METADATA(IID "org.kde.kio.worker.man" FILE "man.json")
 };
 
 MANProtocol *MANProtocol::s_self = nullptr;
@@ -113,7 +108,7 @@ static bool parseUrl(const QString &_url, QString &title, QString &section)
 
 
 MANProtocol::MANProtocol(const QByteArray &pool_socket, const QByteArray &app_socket)
-    : QObject(), SlaveBase("man", pool_socket, app_socket)
+    : QObject(), WorkerBase("man", pool_socket, app_socket)
 {
     Q_ASSERT(s_self==nullptr);
     s_self = this;
@@ -425,7 +420,7 @@ extern void output_real(const char *insert)
 
 //---------------------------------------------------------------------
 
-void MANProtocol::get(const QUrl &url)
+KIO::WorkerResult MANProtocol::get(const QUrl &url)
 {
     qCDebug(KIO_MAN_LOG) << "GET " << url.url();
 
@@ -437,8 +432,7 @@ void MANProtocol::get(const QUrl &url)
     if (!parseUrl(url.path(), title, section))
     {
         showMainIndex();
-        finished();
-        return;
+        return KIO::WorkerResult::pass();
     }
 
     // see if an index was requested
@@ -448,8 +442,7 @@ void MANProtocol::get(const QUrl &url)
             showMainIndex();
         else
             showIndex(section);
-        finished();
-        return;
+        return KIO::WorkerResult::pass();
     }
 
     QStringList foundPages = findPages(section, title);
@@ -464,7 +457,7 @@ void MANProtocol::get(const QUrl &url)
                            "for man pages, either using the <envar>MANPATH</envar> environment "
                            "variable or a configuration file in the <filename>/etc</filename> "
                            "directory.", title.toHtmlEscaped()));
-        return;
+        return KIO::WorkerResult::pass();
     }
 
     // Sort the list of pages now, for display if required and for
@@ -487,8 +480,7 @@ void MANProtocol::get(const QUrl &url)
                 // allowing for a compression suffix.  Output the list of multiple
                 // pages only.
                 outputMatchingPages(foundPages);
-                finished();
-                return;
+                return KIO::WorkerResult::pass();
             }
         }
     }
@@ -497,7 +489,10 @@ void MANProtocol::get(const QUrl &url)
     m_outputBuffer.open(QIODevice::WriteOnly);
     const QByteArray filename = QFile::encodeName(pageFound);
     const char *buf = readManPage(filename);
-    if (buf==nullptr) return;
+    if (!buf) {
+        // readManPage emits an error page instead, so still passing
+        return KIO::WorkerResult::pass();;
+    }
 
     // will call output_real
     scan_man_page(buf);
@@ -511,7 +506,7 @@ void MANProtocol::get(const QUrl &url)
 
     // tell we are done
     data(QByteArray());
-    finished();
+    return KIO::WorkerResult::pass();
 }
 
 //---------------------------------------------------------------------
@@ -585,11 +580,7 @@ char *MANProtocol::readManPage(const char *_filename)
             qCDebug(KIO_MAN_LOG) << "resolved to " << filename;
         }
 
-#if KARCHIVE_VERSION >= QT_VERSION_CHECK(5, 85, 0)
         KCompressionDevice fd(QFile::encodeName(filename));
-#else
-        KFilterDev fd(QFile::encodeName(filename));
-#endif
 
         if (!fd.open(QIODevice::ReadOnly)) {
             outputError(xi18nc("@info", "The man page <filename>%1</filename> could not be read.", QFile::decodeName(filename)));
@@ -617,7 +608,9 @@ void MANProtocol::outputHeader(QTextStream &os, const QString &header, const QSt
 {
     const QString pageTitle = (!title.isEmpty() ? title : header);
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     os.setCodec("UTF-8");
+#endif
 
     // The same header and styling as generated by man2html
     os << "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";
@@ -656,9 +649,9 @@ void MANProtocol::outputFooter(QTextStream &os)
 }
 
 
-// This calls SlaveBase::finished(), so do not call any other
-// SlaveBase functions afterwards.  It is assumed that mimeType()
-// has already been called at the start of get().
+// Do not call any other WorkerBase functions afterwards,
+// only return a KIO::WorkerResult::pass().
+// It is assumed that mimeType() has already been called at the start of get().
 void MANProtocol::outputError(const QString& errmsg)
 {
     QByteArray array;
@@ -670,7 +663,6 @@ void MANProtocol::outputError(const QString& errmsg)
 
     data(array);
     data(QByteArray());
-    finished();
 }
 
 
@@ -701,7 +693,7 @@ void MANProtocol::outputMatchingPages(const QStringList &matchingPages)
 }
 
 
-void MANProtocol::stat( const QUrl& url)
+KIO::WorkerResult MANProtocol::stat( const QUrl& url)
 {
     qCDebug(KIO_MAN_LOG) << "STAT " << url.url();
 
@@ -709,8 +701,7 @@ void MANProtocol::stat( const QUrl& url)
 
     if (!parseUrl(url.path(), title, section))
     {
-        error(KIO::ERR_MALFORMED_URL, url.url());
-        return;
+        return KIO::WorkerResult::fail(KIO::ERR_MALFORMED_URL, url.url());
     }
 
     qCDebug(KIO_MAN_LOG) << "URL" << url.url() << "parsed to title" << title << "section" << section;
@@ -722,7 +713,8 @@ void MANProtocol::stat( const QUrl& url)
     entry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("text/html"));
 
     statEntry(entry);
-    finished();
+
+    return KIO::WorkerResult::pass();
 }
 
 
@@ -742,8 +734,8 @@ extern "C"
             exit(-1);
         }
 
-        MANProtocol slave(argv[2], argv[3]);
-        slave.dispatchLoop();
+        MANProtocol worker(argv[2], argv[3]);
+        worker.dispatchLoop();
 
         qCDebug(KIO_MAN_LOG) << "Done";
 
@@ -752,10 +744,11 @@ extern "C"
 
 }
 
-void MANProtocol::mimetype(const QUrl & /*url*/)
+KIO::WorkerResult MANProtocol::mimetype(const QUrl & /*url*/)
 {
     mimeType("text/html");
-    finished();
+
+    return KIO::WorkerResult::pass();
 }
 
 //---------------------------------------------------------------------
@@ -1125,7 +1118,9 @@ void MANProtocol::showIndex(const QString& section)
 
     QByteArray array_d;
     QTextStream os(&array_d, QIODevice::WriteOnly);
-    os.setCodec( "UTF-8" );
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    os.setCodec("UTF-8");
+#endif
     os << "<div class=\"secidxmain\">\n";
 
     // compose list of search paths -------------------------------------------------------------
@@ -1319,7 +1314,7 @@ void MANProtocol::showIndex(const QString& section)
     // Do not call finished(), the caller will do that
 }
 
-void MANProtocol::listDir(const QUrl &url)
+KIO::WorkerResult MANProtocol::listDir(const QUrl &url)
 {
     qCDebug(KIO_MAN_LOG) << url;
 
@@ -1327,8 +1322,7 @@ void MANProtocol::listDir(const QUrl &url)
     QString section;
 
     if ( !parseUrl(url.path(), title, section) ) {
-        error( KIO::ERR_MALFORMED_URL, url.url() );
-        return;
+        return KIO::WorkerResult::fail( KIO::ERR_MALFORMED_URL, url.url() );
     }
 
     // stat() and listDir() declared that everything is a HTML file.
@@ -1336,13 +1330,12 @@ void MANProtocol::listDir(const QUrl &url)
     // But we cannot list man:ls as a directory, this makes no sense (#154173).
 
     if (!title.isEmpty() && title != "/") {
-        error(KIO::ERR_IS_FILE, url.url());
-        return;
+        return KIO::WorkerResult::fail(KIO::ERR_IS_FILE, url.url());
     }
 
     // There is no need to accumulate a list of UDSEntry's and deliver
-    // them all in one block with SlaveBase::listEntries(), because
-    // SlaveBase::listEntry() batches the entries and delivers them
+    // them all in one block with WorkerBase::listEntries(), because
+    // WorkerBase::listEntry() batches the entries and delivers them
     // timed to maximise application performance.
 
     UDSEntry uds_entry;
@@ -1370,15 +1363,21 @@ void MANProtocol::listDir(const QUrl &url)
         {
             // Remove any compression suffix present
             QString name = stripCompression(page);
+            QString displayName;
             // Remove any preceding pathname components, just leave the base name
             int pos = name.lastIndexOf('/');
             if (pos>0) name = name.mid(pos+1);
             // Remove the section suffix
             pos = name.lastIndexOf('.');
-            if (pos>0) name.truncate(pos);
+            if (pos>0)
+            {
+                displayName = name.left(pos)+" ("+name.mid(pos+1)+')';
+                name.truncate(pos);
+            }
 
             uds_entry.clear();
             uds_entry.fastInsert(KIO::UDSEntry::UDS_NAME, name);
+            if (!displayName.isEmpty()) uds_entry.fastInsert(KIO::UDSEntry::UDS_DISPLAY_NAME, displayName);
             uds_entry.fastInsert(KIO::UDSEntry::UDS_URL, ("man:" + page));
             uds_entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
             uds_entry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("text/html"));
@@ -1386,7 +1385,7 @@ void MANProtocol::listDir(const QUrl &url)
         }
     }
 
-    finished();
+    return KIO::WorkerResult::pass();
 }
 
 
